import chimera, numpy, Tkinter as tk, matplotlib.pyplot as plt
from chimera.baseDialog import ModelessDialog
from numpy import arange
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
from matplotlib.figure import Figure

from StructBio.EnergyFunctions.LJ import LJ_evaluation

# Dictionary of SCWRL atomic radii [JMB (1997) 267, 1268-1282 Table 5.]
stericClashRadii_D = {"N": 1.1, "CT": 1.3, "C": 1.3, "O": 1.1,
                      "O2": 0.7, "N3": 0.4, "CA": 0.8, "N2": 0.4}

#-----------------------------------------------------------------------
# Function to compute steric clash "energy value" given atom codes and separation distance d.
def stericClash(aCode, bCode, d):
    r0 = stericClashRadii_D[aCode] + stericClashRadii_D[bCode]
    if d > r0: return 0.0
    if d < 0.83*r0: return 10.0
    return 57.273*(1.0 - d/r0)

#-----------------------------------------------------------------------
# Function to compute LJ energy given atom codes and separation distance d.
def pairEnergyLJ(aCode, bCode, d):
    if d > 10.0: return 0.0  # Return 0 if interatomic distance > 10. (see Kingsford thesis).
    d_2 = d * d; d_6 = d_2 * d_2 * d_2; d_12 = d_6 * d_6
    pairEnergy = energyLJ.a_ij_D[aCode + "#" + bCode]/d_12 - energyLJ.b_ij_D[aCode + "#" + bCode]/d_6
    if abs(pairEnergy) < 0.000001: pairEnergy = 0.0
    return min(pairEnergy, 20.0)

#=======================================================================
class TkPlusPlot(ModelessDialog):
    buttons = ("Save", "Close")
    help = False    # Leave this out to get a Help button.
    title = "Comparing Lennard-Jones & Piecewise Linear Energy Functions"

    def fillInUI(self, parent):
        # Text to be displayed in the dialog window:
        leftFrame = tk.Frame(parent, bd = 2, relief = tk.RIDGE)
        leftFrame.grid(row = 0, column = 0, padx = 4, pady = 4, sticky = tk.W)
        topText = tk.Label(leftFrame, text =  " Pick two atom types:",
                           font = ("Helvetica", 11, "normal"))
        topText.grid(row = 0, column = 0, columnspan = 2, pady = 4, sticky = tk.W)

        self.rBtn_a_Result = tk.StringVar() # Use tk.IntVar() here for integer result values.
        # Note that the result codes are appropriate as keys for the dictionaries
        # in the LJ_evaluation class.
        radioBtn_a_L = [("  Backbone N ", "N",  1, 0),
                        ("  Backbone CA", "CT", 2, 0),
                        ("  Backbone C ", "C",  3, 0),
                        ("  Backbone O ", "O",  4, 0),
                        (" ASP, ASN OD1", "O2", 5, 0),
                        ("    LYS NZ   ", "N3", 6, 0),
                        ("    ARG CZ   ", "CA", 7, 0),
                        ("    ARG NH2  ", "N2", 8, 0)]
        
        self.rBtn_a_Result.set("NO SEL") # This "nonexistent button" starts off selected.
        for text, rslt, rw, cl in radioBtn_a_L:
            rBtn_a = tk.Radiobutton(leftFrame, text = text, variable = self.rBtn_a_Result,
                                    value = rslt, command = lambda: self.rBtnfunc(parent))
            rBtn_a.grid(row = rw, column = cl, padx = 4, sticky = tk.W)

        self.rBtn_b_Result = tk.StringVar()
        radioBtn_b_L = [("", "N",  1, 1),
                        ("", "CT", 2, 1),
                        ("", "C",  3, 1),
                        ("", "O",  4, 1),
                        ("", "O2", 5, 1),
                        ("", "N3", 6, 1),
                        ("", "CA", 7, 1),
                        ("", "N2", 8, 1)]
        
        self.rBtn_b_Result.set("NO SEL") # This "nonexistent button" starts off selected.
        for text, rslt, rw, cl in radioBtn_b_L:
            rBtn_b = tk.Radiobutton(leftFrame, text = text, variable = self.rBtn_b_Result,
                                    value = rslt, command = lambda: self.rBtnfunc(parent))
            rBtn_b.grid(row = rw, column = cl, padx = 4, sticky = tk.W + tk.E)

    #-----------------------------------------------------------------------
    def Save(self):
        # Save file.
        if self.fig != None:
            self.fig.savefig(self.a + "_" + self.b + "_LJvsLinear", dpi = self.fig.dpi)

    #-------------------------------------------------------------------------
    # Callback function needed for radio buttons.  The function gets the button
    # selection from each column.  This determines the types of the two atoms
    # involved in the energy functions.
    def rBtnfunc(self, parent):
        if self.rBtn_a_Result.get() == "NO SEL" or self.rBtn_b_Result.get() == "NO SEL":
            return
        #-----------------------------------------------------------------------
        # Do the plot:
        self.fig = Figure(figsize = (5,6), dpi = 100)
        ax = self.fig.add_subplot(111, aspect = 0.25)
        ax.set_title("Piecewise Linear & LJ", fontsize = 14)
        ax.set_xlabel("r / r0", fontsize = 12, style = 'italic')
        ax.set_ylabel("Energy", fontsize = 12, style = 'italic')

        ax.set_xlim([0.0, 2.0])
        ax.set_ylim([-1.0, 11.0])
        ax.set_xticks([x for x in arange(0.0, 2.1, 0.5)])
        ax.set_yticks([y for y in arange(-1.0, 12.0, 1.)])
        ax.grid(True)
        ax.hlines(0.0, 0.0, 2.0, colors = 'k', linestyles = 'dashed')
        #ax.vlines(0.0, -1., 11.0, colors = 'k', linestyles = 'solid')

        self.a = self.rBtn_a_Result.get()
        self.b = self.rBtn_b_Result.get()
        
        # Let ratio be r/r0 where r0 is the sum of the LJ radii.
        # Build a list of ratio values:
        ratio_L = arange(0.1, 2.0+0.01, 0.01)
        #Lennard-Jones r0 value:
        r0LJ = energyLJ.radiusMap_D[self.a] + energyLJ.radiusMap_D[self.b]
        # So separation distance r is the ratio times r0:
        r_L = ratio_L * r0LJ
        
        eLJ_L = [pairEnergyLJ(self.a, self.b, d) for d in r_L]
        ax.plot(ratio_L, eLJ_L)

        # Steric Clash. We use the separation distances as per the LJ plot:
        eSC = [stericClash(self.a, self.b, d) for d in r_L]
        ax.plot(ratio_L, eSC)

        #-------------------------------------------------------------------------
        # Set up a canvas and put the Figure into it.
        rightFrame = tk.Frame(parent, bd = 2, relief = tk.RIDGE)
        rightFrame.grid(row = 0, column = 1, padx = 4, pady = 4, sticky = tk.E)

        canvas = FigureCanvasTkAgg(self.fig, master = rightFrame)
        canvas.get_tk_widget().grid(row = 0, column = 0)
            
#===============================================================================
# Mainline
#===============================================================================
energyLJ = LJ_evaluation()

wb = TkPlusPlot()


